# Copyright (c) 2016 Huawei Technologies India Pvt.Limited.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from osc_lib.tests import utils as tests_utils

from networking_sfc.osc import common
from networking_sfc.osc.flow_classifier import flow_classifier
from networking_sfc.tests.unit.osc import fakes


def _get_id(client, id_or_name, resource):
    return id_or_name


class TestListFlowClassifierList(fakes.TestNeutronClientOSCV2):

    _fc = fakes.FakeFlowClassifier.create_flow_classifiers(count=1)
    columns = ('ID', 'Name', 'Protocol', 'Source-IP', 'Destination-IP',
               'Logical-Source-Port', 'Logical-Destination-Port')
    data = []
    _flow_classifier = _fc['flow_classifiers'][0]
    data.append((
        _flow_classifier['id'],
        _flow_classifier['name'],
        _flow_classifier['protocol'],
        _flow_classifier['source_ip_prefix'],
        _flow_classifier['destination_ip_prefix'],
        _flow_classifier['logical_source_port'],
        _flow_classifier['logical_destination_port']))
    _flow_classifier1 = {'flow_classifiers': _flow_classifier}
    _flow_classifier_id = _flow_classifier['id']

    def setUp(self):
        super(TestListFlowClassifierList, self).setUp()

        self.neutronclient.list_ext = mock.Mock(
            return_value=self._flow_classifier1
        )
        # Get the command object to test
        self.cmd = flow_classifier.ListFlowClassifier(self.app, self.namespace)

    def test_flow_classifier_list(self):
        client = self.app.client_manager.neutronclient
        mock_flow_classifier_list = client.list_ext
        parsed_args = self.check_parser(self.cmd, [], [])
        columns = self.cmd.take_action(parsed_args)
        data = mock_flow_classifier_list.assert_called_once_with(
            collection='flow_classifiers', path='/sfc/flow_classifiers',
            retrieve_all=True)
        self.assertEqual(self.columns, columns[0])
        self.assertIsNone(data)


class TestCreateFlowClassifier(fakes.TestNeutronClientOSCV2):
    # The new port_pair created
    _flow_classifier = fakes.FakeFlowClassifier.create_flow_classifier()

    columns = (
        'id',
        'name',
        'description',
        'protocol',
        'source_ip_prefix',
        'destination_ip_prefix',
        'logical_source_port',
        'logical_destiantion_port',
        'source_port_min',
        'source_port_max',
        'destination_port_min',
        'destination_port_max',
        'l7_parameters',
    )

    def get_data(self):
        return (
            self._flow_classifier['id'],
            self._flow_classifier['name'],
            self._flow_classifier['description'],
            self._flow_classifier['protocol'],
            self._flow_classifier['source_ip_prefix'],
            self._flow_classifier['destination_ip_prefix'],
            self._flow_classifier['logical_source_port'],
            self._flow_classifier['logical_destination_port'],
            self._flow_classifier['source_port_range_min'],
            self._flow_classifier['source_port_range_max'],
            self._flow_classifier['destination_port_range_min'],
            self._flow_classifier['destination_port_range_max'],
            self._flow_classifier['l7_parameters'],
        )

    def setUp(self):
        super(TestCreateFlowClassifier, self).setUp()
        mock.patch('networking_sfc.osc.common.find_sfc_resource',
                   new=_get_id).start()
        common.create_sfc_resource = mock.Mock(
            return_value={'flow_classifiers': self._flow_classifier})
        self.data = self.get_data()

        # Get the command object to test
        self.cmd = flow_classifier.CreateFlowClassifier(self.app,
                                                        self.namespace)

    def test_create_flow_classifier(self):
        arglist = []
        verifylist = []

        self.assertRaises(tests_utils.ParserException, self.check_parser,
                          self.cmd, arglist, verifylist)


class TestDeleteFlowClassifier(fakes.TestNeutronClientOSCV2):

    def setUp(self):
        super(TestDeleteFlowClassifier, self).setUp()
        _flow_classifier = fakes.FakePortPairGroup.create_port_pair_groups()
        self._flow_classifier = _flow_classifier['port_pair_groups'][0]
        _port_pair_group_id = self._flow_classifier['id']
        common.delete_sfc_resource = mock.Mock(return_value=None)
        common.find_sfc_resource = mock.Mock(return_value=_port_pair_group_id)
        self.cmd = flow_classifier.DeleteFlowClassifier(self.app,
                                                        self.namespace)

    def test_delete_port_pair_group(self):
        client = self.app.client_manager.neutronclient
        fc_id = self._flow_classifier['id']
        mock_flow_classifier_delete = common.delete_sfc_resource
        arglist = [
            self._flow_classifier['id'],
        ]
        verifylist = [
            ('flow_classifier', self._flow_classifier['id']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        mock_flow_classifier_delete.assert_called_once_with(client,
                                                            'flow_classifier',
                                                            fc_id)
        self.assertIsNone(result)


class TestShowFlowClassifier(fakes.TestNeutronClientOSCV2):

    _fc = fakes.FakeFlowClassifier.create_flow_classifier()
    data = (
        _fc['description'],
        _fc['destination_ip_prefix'],
        _fc['destination_port_range_max'],
        _fc['destination_port_range_min'],
        _fc['ethertype'],
        _fc['id'],
        _fc['l7_parameters'],
        _fc['logical_destination_port'],
        _fc['logical_source_port'],
        _fc['name'],
        _fc['no_flow_classiifier'],
        _fc['project_id'],
        _fc['protocol'],
        _fc['source_ip_prefix'],
        _fc['source_port_range_max'],
        _fc['source_port_range_min']
    )
    _flow_classifier = {'flow_classifier': _fc}
    _flow_classifier_id = _fc['id']
    columns = (
        'description',
        'destination_ip_prefix',
        'destination_port_range_max',
        'destination_port_range_min',
        'ethertype',
        'id',
        'l7_parameters',
        'logical_destination_port',
        'logical_source_port',
        'name',
        'no_flow_classiifier',
        'project_id',
        'protocol',
        'source_ip_prefix',
        'source_port_range_max',
        'source_port_range_min'
    )

    def setUp(self):
        super(TestShowFlowClassifier, self).setUp()
        common.find_sfc_resource = mock.Mock(
            return_value=self._flow_classifier_id)
        common.show_sfc_resource = mock.Mock(
            return_value=self._flow_classifier
        )
        # Get the command object to test
        self.cmd = flow_classifier.ShowFlowClassifier(self.app, self.namespace)

    def test_port_pair_group_show(self):
        client = self.app.client_manager.neutronclient
        fc_id = self._flow_classifier_id
        mock_flow_classifier_show = common.show_sfc_resource
        arglist = [
            self._flow_classifier_id,
        ]
        verifylist = [
            ('flow_classifier', self._flow_classifier_id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        data = self.cmd.take_action(parsed_args)
        mock_flow_classifier_show.assert_called_once_with(client,
                                                          'flow_classifier',
                                                          fc_id)
        self.assertEqual(self.columns, data[0])
        self.assertEqual(self.data, data[1])


class TestUpdateFlowClassifier(fakes.TestNeutronClientOSCV2):
    _flow_classifier = fakes.FakeFlowClassifier.create_flow_classifier()
    _flow_classifier_name = _flow_classifier['name']
    _flow_classifier_id = _flow_classifier['id']

    def setUp(self):
        super(TestUpdateFlowClassifier, self).setUp()
        common.update_sfc_resource = mock.Mock(return_value=None)
        common.find_sfc_resource = mock.Mock(
            return_value=self._flow_classifier_id)
        self.cmd = flow_classifier.UpdateFlowClassifier(
            self.app, self.namespace)

    def test_update_flow_classifier(self):
        client = self.app.client_manager.neutronclient
        fc_id = self._flow_classifier_id
        mock_flow_classifier_update = common.update_sfc_resource
        arglist = [
            self._flow_classifier_name,
            '--name', 'name_updated',
            '--description', 'desc_updated'
        ]
        verifylist = [
            ('flow_classifier', self._flow_classifier_name),
            ('name', 'name_updated'),
            ('description', 'desc_updated'),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        attrs = {
            'name': 'name_updated',
            'description': 'desc_updated'}
        mock_flow_classifier_update.assert_called_once_with(client,
                                                            'flow_classifier',
                                                            attrs,
                                                            fc_id)

        self.assertIsNone(result)



# Copyright (c) 2016 Huawei Technologies India Pvt.Limited.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from osc_lib.command import command
from osc_lib import utils

from neutronclient._i18n import _
from neutronclient.common import utils as nc_utils

from networking_sfc.osc import common

LOG = logging.getLogger(__name__)

SFC_COMMON_PREFIX = "/sfc"
PORT_CHAIN_PATH = SFC_COMMON_PREFIX + "/port_chains"
resource = 'port_chain'


class CreatePortChain(command.ShowOne):
    """Create a Port Chain."""

    def get_parser(self, prog_name):
        parser = super(CreatePortChain, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='NAME',
            help=_('Name of the Port Chain.'))
        parser.add_argument(
            '--description',
            help=_('Description for the Port Chain.'))
        parser.add_argument(
            '--port-pair-group',
            metavar='PORT-PAIR-GROUP',
            dest='port_pair_groups',
            required=True,
            action='append',
            help=_('ID or name of the Port Pair Group. '
                   'This option can be repeated.'))
        parser.add_argument(
            '--flow-classifier',
            default=[],
            metavar='FLOW-CLASSIFIER',
            dest='flow_classifiers',
            action='append',
            help=_('ID or name of the Flow Classifier.'
                   'This option can be repeated.'))
        parser.add_argument(
            '--chain-parameters',
            metavar='[correlation=CORRELATION_TYPE, symmetric=BOOLEAN_TYPE]',
            type=nc_utils.str2dict_type(optional_keys=['correlation',
                                                       'symmetric']),
            help=_('Dictionary of chain parameters. Supports '
                   'correlation=mpls and symmetric=true|false'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        attrs = _get_common_attrs(self.app.client_manager, parsed_args)
        obj = common.create_sfc_resource(client, resource, attrs)
        columns = common.get_columns(obj[resource])
        data = utils.get_dict_properties(obj[resource], columns)
        return columns, data


class UpdatePortChain(command.Command):
    """Update Port Chain's information."""

    def get_parser(self, prog_name):
        parser = super(UpdatePortChain, self).get_parser(prog_name)
        parser.add_argument(
            '--name',
            metavar='NAME',
            help=_('Name of the Port Chain.'))
        parser.add_argument(
            '--description',
            help=_('Description for the Port Chain.'))
        fw_args = parser.add_mutually_exclusive_group()
        fw_args.add_argument(
            '--flow-classifier',
            metavar='FLOW-CLASSIFIER',
            dest='flow_classifiers',
            action='append',
            help=_('ID or name of the Flow Classifier. '
                   'This option can be repeated.'))
        fw_args.add_argument(
            '--no-flow-classifier',
            action='store_true',
            help=_('Associate no Flow Classifier with the Port Chain.'))
        parser.add_argument(
            '--port-pair-group',
            metavar='PORT-PAIR-GROUP',
            dest='port_pair_groups',
            action='append',
            help=_('ID or name of the port pair group. '
                    'This option can be repeated.'))
        parser.add_argument(
            'port_chain',
            metavar='PORT-CHAIN',
            help=_("ID or name of the Port Chain to update."))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        id = common.find_sfc_resource(client,
                                      resource,
                                      parsed_args.port_chain)
        attrs = _get_common_attrs(self.app.client_manager, parsed_args,
                                  is_create=False)
        if parsed_args.no_flow_classifier:
            attrs['flow_classifiers'] = []
        common.update_sfc_resource(client, resource, attrs, id)


class DeletePortChain(command.Command):
    """Delete a given Port Chain."""

    def get_parser(self, prog_name):
        parser = super(DeletePortChain, self).get_parser(prog_name)
        parser.add_argument(
            'port_chain',
            metavar="PORT_CHAIN",
            help=_("ID or name of the Port Chain to delete.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        id = common.find_sfc_resource(client,
                                      resource,
                                      parsed_args.port_chain)
        common.delete_sfc_resource(client, resource, id)


class ListPortChain(command.Lister):
    """List Port Chains."""

    def take_action(self, parsed_args):
        data = self.app.client_manager.neutronclient.list_ext(
            collection='port_chains', path=PORT_CHAIN_PATH,
            retrieve_all=True)
        headers = ('ID', 'Name', 'Port Pair Groups', 'Flow Classifiers',
                   'Chain Parameters')
        columns = ('id', 'name', 'port_pair_groups', 'flow_classifiers',
                   'chain_parameters')
        return (headers,
                (utils.get_dict_properties(
                    s, columns,
                ) for s in data['port_chains']))


class ShowPortChain(command.ShowOne):
    """Show information of a given Port Pair Groups."""

    def get_parser(self, prog_name):
        parser = super(ShowPortChain, self).get_parser(prog_name)
        parser.add_argument(
            'port_chain',
            metavar="PORT_CHAIN",
            help=_("ID or name of the Port Chain to display.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        id = common.find_sfc_resource(client,
                                      resource,
                                      parsed_args.port_chain)
        obj = common.show_sfc_resource(client, resource, id)
        columns = common.get_columns(obj[resource])
        data = utils.get_dict_properties(obj[resource], columns)
        return columns, data


def _get_common_attrs(client_manager, parsed_args, is_create=True):
    attrs = {}
    if parsed_args.name is not None:
        attrs['name'] = str(parsed_args.name)
    if parsed_args.description is not None:
        attrs['description'] = str(parsed_args.description)
    if ('port_pair_groups' in parsed_args and
            parsed_args.port_pair_groups is not None):
        attrs['port_pair_groups'] = [(common.find_sfc_resource(
                                      client_manager.neutronclient,
                                      'port_pair_group', ppg))
                                     for ppg in parsed_args.port_pair_groups]
    if ('flow_classifiers' in parsed_args and
            parsed_args.flow_classifiers is not None):
        attrs['flow_classifiers'] = [(common.find_sfc_resource(
                                      client_manager.neutronclient,
                                      'flow_classifier', fc))
                                     for fc in parsed_args.flow_classifiers]
    if is_create is True:
        _get_attrs(attrs, parsed_args)
    return attrs


def _get_attrs(attrs, parsed_args):
    if 'chain_parameters' in parsed_args:
        attrs['chain_parameters'] = parsed_args.chain_parameters



# Copyright (c) 2016 Huawei Technologies India Pvt.Limited.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from osc_lib.command import command
from osc_lib import utils

from neutronclient._i18n import _
from neutronclient.common import utils as nc_utils

from networking_sfc.osc import common

LOG = logging.getLogger(__name__)

SFC_COMMON_PREFIX = "/sfc"
PORT_PAIR_GROUP_PATH = SFC_COMMON_PREFIX + "/port_pair_groups"
resource = 'port_pair_group'


class CreatePortPairGroup(command.ShowOne):
    """Create a Port Pair Group."""

    def get_parser(self, prog_name):
        parser = super(CreatePortPairGroup, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='NAME',
            help=_('Name of the Port Pair Group.'))
        parser.add_argument(
            '--description',
            help=_('Description for the Port Pair Group.'))
        parser.add_argument(
            '--port-pair',
            metavar='PORT-PAIR',
            dest='port_pairs',
            default=[],
            action='append',
            help=_('ID or name of the Port Pair.'
                   'This option can be repeated.'))
        parser.add_argument(
            '--port-pair-group-parameters',
            metavar='[lb_fields=LB_FIELDS]',
            type=nc_utils.str2dict_type(optional_keys=['lb_fields']),
            help=_('Dictionary of Port pair group parameters. '
                   'Currently, only \'&\' separated string of the lb_fields '
                   'are supported.'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        attrs = _get_common_attrs(self.app.client_manager, parsed_args)
        obj = common.create_sfc_resource(client, resource, attrs)
        columns = common.get_columns(obj[resource])
        data = utils.get_dict_properties(obj[resource], columns)
        return columns, data


class UpdatePortPairGroup(command.Command):
    """Update Port Pair Group's information."""

    def get_parser(self, prog_name):
        parser = super(UpdatePortPairGroup, self).get_parser(prog_name)
        parser.add_argument(
            '--name',
            metavar='NAME',
            help=_('Name of the Port Pair Group.'))
        parser.add_argument(
            '--description',
            help=_('Description for the Port Pair Group.'))
        parser.add_argument(
            '--port-pair',
            metavar="PORT_PAIR",
            dest='port_pairs',
            default=[],
            action='append',
            help=_('ID or name of the Port Pair.'
                   'This option can be repeated.'))
        parser.add_argument(
            'port_pair_group',
            metavar="PORT_PAIR_GROUP",
            help=_("ID or name of the Port Pair Group to update."))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        id = common.find_sfc_resource(client,
                                      resource,
                                      parsed_args.port_pair_group)
        attrs = _get_common_attrs(self.app.client_manager, parsed_args,
                                  is_create=False)
        common.update_sfc_resource(client, resource, attrs, id)


class DeletePortPairGroup(command.Command):
    """Delete a given Port Pair Group."""

    def get_parser(self, prog_name):
        parser = super(DeletePortPairGroup, self).get_parser(prog_name)
        parser.add_argument(
            'port_pair_group',
            metavar="PORT_PAIR_GROUP",
            help=_("ID or name of the Port Pair Group to delete.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        id = common.find_sfc_resource(client,
                                      resource,
                                      parsed_args.port_pair_group)
        common.delete_sfc_resource(client, resource, id)


class ListPortPairGroup(command.Lister):
    """List Port Pair Groups."""

    def take_action(self, parsed_args):
        data = self.app.client_manager.neutronclient.list_ext(
            collection='port_pair_groups', path=PORT_PAIR_GROUP_PATH,
            retrieve_all=True)
        headers = ('ID', 'Name', 'Port Pair', 'Port Pair Group Parameters')
        columns = ('id', 'name', 'port_pairs', 'port_pair_group_parameters')
        return (headers,
                (utils.get_dict_properties(
                    s, columns,
                ) for s in data['port_pair_groups']))


class ShowPortPairGroup(command.ShowOne):
    """Show information of a given Port Pair Groups."""

    def get_parser(self, prog_name):
        parser = super(ShowPortPairGroup, self).get_parser(prog_name)
        parser.add_argument(
            'port_pair_group',
            metavar="PORT_PAIR_GROUP",
            help=_("ID or name of the Port Pair Group to display.)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        id = common.find_sfc_resource(client,
                                      resource,
                                      parsed_args.port_pair_group)
        obj = common.show_sfc_resource(client, resource, id)
        columns = common.get_columns(obj[resource])
        data = utils.get_dict_properties(obj[resource], columns)
        return columns, data


def _get_ppg_param(attrs, ppg):
    attrs['port_pair_group_parameters'] = {}
    for key, value in ppg.items():
        if key == 'lb_fields':
            attrs['port_pair_group_parameters'][key] = (
                [field for field in value.split('&') if field])
        else:
            attrs['port_pair_group_parameters'][key] = value
    return attrs['port_pair_group_parameters']


def _get_common_attrs(client_manager, parsed_args, is_create=True):
    attrs = {}
    if parsed_args.name is not None:
        attrs['name'] = str(parsed_args.name)
    if parsed_args.description is not None:
        attrs['description'] = str(parsed_args.description)
    if parsed_args.port_pairs:
        attrs['port_pairs'] = [common.find_sfc_resource(
            client_manager.neutronclient, 'port_pair', pp)
            for pp in parsed_args.port_pairs]
    if is_create is True:
        _get_attrs(attrs, parsed_args)
    return attrs


def _get_attrs(attrs, parsed_args):
    if ('port_pair_group_parameters' in parsed_args and
            parsed_args.port_pair_group_parameters is not None):
        attrs['port_pair_group_parameters'] = (
            _get_ppg_param(attrs, parsed_args.port_pair_group_parameters))



# Copyright 2015 Futurewei. All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import mock
from webob import exc
import webtest

from neutron_lib import constants as const
from oslo_utils import uuidutils

from neutron.api.v2 import resource as api_res_log
from neutron.common import config
from neutron import manager
from neutron.notifiers import nova as nova_log

from neutron.tests.unit.api.v2 import test_base as test_api_v2
from neutron.tests.unit.extensions import base as test_api_v2_extension

from networking_sfc.extensions import flowclassifier as fc_ext

_uuid = uuidutils.generate_uuid
_get_path = test_api_v2._get_path

FLOW_CLASSIFIER_PATH = (fc_ext.FLOW_CLASSIFIER_PREFIX[1:] + '/' +
                        fc_ext.FLOW_CLASSIFIER_EXT + 's')


class FlowClassifierExtensionTestCase(
    test_api_v2_extension.ExtensionTestCase
):
    fmt = 'json'

    def setUp(self):
        self._mock_unnecessary_logging()
        super(FlowClassifierExtensionTestCase, self).setUp()
        self._setUpExtension(
            'networking_sfc.extensions.flowclassifier.'
            'FlowClassifierPluginBase',
            fc_ext.FLOW_CLASSIFIER_EXT,
            fc_ext.RESOURCE_ATTRIBUTE_MAP,
            fc_ext.Flowclassifier,
            fc_ext.FLOW_CLASSIFIER_PREFIX[1:],
            plural_mappings={}
        )

    def _mock_unnecessary_logging(self):
        mock_log_cfg_p = mock.patch.object(config, 'LOG')
        self.mock_log_cfg = mock_log_cfg_p.start()

        mock_log_manager_p = mock.patch.object(manager, 'LOG')
        self.mock_log_manager = mock_log_manager_p.start()

        mock_log_nova_p = mock.patch.object(nova_log, 'LOG')
        self.mock_log_nova = mock_log_nova_p.start()

        mock_log_api_res_log_p = mock.patch.object(api_res_log, 'LOG')
        self.mock_log_api_res_log = mock_log_api_res_log_p.start()

    def _get_expected_flow_classifier(self, data):
        source_port_range_min = data['flow_classifier'].get(
            'source_port_range_min')
        if source_port_range_min is not None:
            source_port_range_min = int(source_port_range_min)
        source_port_range_max = data['flow_classifier'].get(
            'source_port_range_max')
        if source_port_range_max is not None:
            source_port_range_max = int(source_port_range_max)
        destination_port_range_min = data['flow_classifier'].get(
            'destination_port_range_min')
        if destination_port_range_min is not None:
            destination_port_range_min = int(destination_port_range_min)
        destination_port_range_max = data['flow_classifier'].get(
            'destination_port_range_max')
        if destination_port_range_max is not None:
            destination_port_range_max = int(destination_port_range_max)

        return {'flow_classifier': {
            'name': data['flow_classifier'].get('name') or '',
            'description': data['flow_classifier'].get('description') or '',
            'tenant_id': data['flow_classifier']['tenant_id'],
            'project_id': data['flow_classifier']['project_id'],
            'source_port_range_min': source_port_range_min,
            'source_port_range_max': source_port_range_max,
            'destination_port_range_min': destination_port_range_min,
            'destination_port_range_max': destination_port_range_max,
            'l7_parameters': data['flow_classifier'].get(
                'l7_parameters') or {},
            'destination_ip_prefix': data['flow_classifier'].get(
                'destination_ip_prefix'),
            'source_ip_prefix': data['flow_classifier'].get(
                'source_ip_prefix'),
            'logical_source_port': data['flow_classifier'].get(
                'logical_source_port'),
            'logical_destination_port': data['flow_classifier'].get(
                'logical_destination_port'),
            'ethertype': data['flow_classifier'].get(
                'ethertype') or 'IPv4',
            'protocol': data['flow_classifier'].get(
                'protocol')
        }}

    def test_create_flow_classifier(self):
        flowclassifier_id = _uuid()
        tenant_id = _uuid()
        data = {'flow_classifier': {
            'logical_source_port': _uuid(),
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        expected_data = self._get_expected_flow_classifier(data)
        return_value = copy.copy(expected_data['flow_classifier'])
        return_value.update({'id': flowclassifier_id})
        instance = self.plugin.return_value
        instance.create_flow_classifier.return_value = return_value
        res = self.api.post(
            _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)
        instance.create_flow_classifier.assert_called_with(
            mock.ANY,
            flow_classifier=expected_data)
        self.assertEqual(exc.HTTPCreated.code, res.status_int)
        res = self.deserialize(res)
        self.assertIn('flow_classifier', res)
        self.assertEqual(return_value, res['flow_classifier'])

    def test_create_flow_classifier_source_port_range(self):
        for source_port_range_min in [None, 100, '100']:
            for source_port_range_max in [None, 200, '200']:
                flowclassifier_id = _uuid()
                tenant_id = _uuid()
                data = {'flow_classifier': {
                    'source_port_range_min': source_port_range_min,
                    'source_port_range_max': source_port_range_max,
                    'logical_source_port': _uuid(),
                    'tenant_id': tenant_id, 'project_id': tenant_id,
                }}
                expected_data = self._get_expected_flow_classifier(data)
                return_value = copy.copy(expected_data['flow_classifier'])
                return_value.update({'id': flowclassifier_id})
                instance = self.plugin.return_value
                instance.create_flow_classifier.return_value = return_value
                res = self.api.post(
                    _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt),
                    self.serialize(data),
                    content_type='application/%s' % self.fmt)
                instance.create_flow_classifier.assert_called_with(
                    mock.ANY,
                    flow_classifier=expected_data)
                self.assertEqual(exc.HTTPCreated.code, res.status_int)
                res = self.deserialize(res)
                self.assertIn('flow_classifier', res)
                self.assertEqual(return_value, res['flow_classifier'])

    def test_create_flow_classifier_destination_port_range(self):
        for destination_port_range_min in [None, 100, '100']:
            for destination_port_range_max in [None, 200, '200']:
                flowclassifier_id = _uuid()
                tenant_id = _uuid()
                data = {'flow_classifier': {
                    'destination_port_range_min': destination_port_range_min,
                    'destination_port_range_max': destination_port_range_max,
                    'logical_source_port': _uuid(),
                    'tenant_id': tenant_id, 'project_id': tenant_id,
                }}
                expected_data = self._get_expected_flow_classifier(data)
                return_value = copy.copy(expected_data['flow_classifier'])
                return_value.update({'id': flowclassifier_id})
                instance = self.plugin.return_value
                instance.create_flow_classifier.return_value = return_value
                res = self.api.post(
                    _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt),
                    self.serialize(data),
                    content_type='application/%s' % self.fmt)
                instance.create_flow_classifier.assert_called_with(
                    mock.ANY,
                    flow_classifier=expected_data)
                self.assertEqual(exc.HTTPCreated.code, res.status_int)
                res = self.deserialize(res)
                self.assertIn('flow_classifier', res)
                self.assertEqual(return_value, res['flow_classifier'])

    def test_create_flow_classifier_source_ip_prefix(self):
        for logical_source_ip_prefix in [
            None, '10.0.0.0/8'
        ]:
            flowclassifier_id = _uuid()
            tenant_id = _uuid()
            data = {'flow_classifier': {
                'source_ip_prefix': logical_source_ip_prefix,
                'logical_source_port': _uuid(),
                'tenant_id': tenant_id, 'project_id': tenant_id,
            }}
            expected_data = self._get_expected_flow_classifier(data)
            return_value = copy.copy(expected_data['flow_classifier'])
            return_value.update({'id': flowclassifier_id})
            instance = self.plugin.return_value
            instance.create_flow_classifier.return_value = return_value
            res = self.api.post(
                _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt),
                self.serialize(data),
                content_type='application/%s' % self.fmt)
            instance.create_flow_classifier.assert_called_with(
                mock.ANY,
                flow_classifier=expected_data)
            self.assertEqual(exc.HTTPCreated.code, res.status_int)
            res = self.deserialize(res)
            self.assertIn('flow_classifier', res)
            self.assertEqual(return_value, res['flow_classifier'])

    def test_create_flow_classifier_destination_ip_prefix(self):
        for logical_destination_ip_prefix in [
            None, '10.0.0.0/8'
        ]:
            flowclassifier_id = _uuid()
            tenant_id = _uuid()
            data = {'flow_classifier': {
                'destination_ip_prefix': logical_destination_ip_prefix,
                'logical_source_port': _uuid(),
                'tenant_id': tenant_id, 'project_id': tenant_id,
            }}
            expected_data = self._get_expected_flow_classifier(data)
            return_value = copy.copy(expected_data['flow_classifier'])
            return_value.update({'id': flowclassifier_id})
            instance = self.plugin.return_value
            instance.create_flow_classifier.return_value = return_value
            res = self.api.post(
                _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt),
                self.serialize(data),
                content_type='application/%s' % self.fmt)
            instance.create_flow_classifier.assert_called_with(
                mock.ANY,
                flow_classifier=expected_data)
            self.assertEqual(res.status_int, exc.HTTPCreated.code)
            res = self.deserialize(res)
            self.assertIn('flow_classifier', res)
            self.assertEqual(return_value, res['flow_classifier'])

    def test_create_flow_classifier_logical_source_port(self):
        for logical_source_port in [
            None, _uuid()
        ]:
            flowclassifier_id = _uuid()
            tenant_id = _uuid()
            data = {'flow_classifier': {
                'logical_source_port': logical_source_port,
                'tenant_id': tenant_id, 'project_id': tenant_id,
            }}
            expected_data = self._get_expected_flow_classifier(data)
            return_value = copy.copy(expected_data['flow_classifier'])
            return_value.update({'id': flowclassifier_id})
            instance = self.plugin.return_value
            instance.create_flow_classifier.return_value = return_value
            res = self.api.post(
                _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt),
                self.serialize(data),
                content_type='application/%s' % self.fmt)
            instance.create_flow_classifier.assert_called_with(
                mock.ANY,
                flow_classifier=expected_data)
            self.assertEqual(exc.HTTPCreated.code, res.status_int)
            res = self.deserialize(res)
            self.assertIn('flow_classifier', res)
            self.assertEqual(return_value, res['flow_classifier'])

    def test_create_flow_classifier_logical_destination_port(self):
        for logical_destination_port in [
            None, _uuid()
        ]:
            flowclassifier_id = _uuid()
            tenant_id = _uuid()
            data = {'flow_classifier': {
                'logical_destination_port': logical_destination_port,
                'tenant_id': tenant_id, 'project_id': tenant_id,
            }}
            expected_data = self._get_expected_flow_classifier(data)
            return_value = copy.copy(expected_data['flow_classifier'])
            return_value.update({'id': flowclassifier_id})
            instance = self.plugin.return_value
            instance.create_flow_classifier.return_value = return_value
            res = self.api.post(
                _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt),
                self.serialize(data),
                content_type='application/%s' % self.fmt)
            instance.create_flow_classifier.assert_called_with(
                mock.ANY,
                flow_classifier=expected_data)
            self.assertEqual(exc.HTTPCreated.code, res.status_int)
            res = self.deserialize(res)
            self.assertIn('flow_classifier', res)
            self.assertEqual(return_value, res['flow_classifier'])

    def test_create_flow_classifier_l7_parameters(self):
        for l7_parameters in [None, {}]:
            flowclassifier_id = _uuid()
            tenant_id = _uuid()
            data = {'flow_classifier': {
                'logical_source_port': _uuid(),
                'tenant_id': tenant_id, 'project_id': tenant_id,
                'l7_parameters': l7_parameters
            }}
            expected_data = self._get_expected_flow_classifier(data)
            return_value = copy.copy(expected_data['flow_classifier'])
            return_value.update({'id': flowclassifier_id})
            instance = self.plugin.return_value
            instance.create_flow_classifier.return_value = return_value
            res = self.api.post(
                _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt),
                self.serialize(data),
                content_type='application/%s' % self.fmt)
            instance.create_flow_classifier.assert_called_with(
                mock.ANY,
                flow_classifier=expected_data)
            self.assertEqual(exc.HTTPCreated.code, res.status_int)
            res = self.deserialize(res)
            self.assertIn('flow_classifier', res)
            self.assertEqual(return_value, res['flow_classifier'])

    def test_create_flow_classifier_ethertype(self):
        for ethertype in [None, 'IPv4', 'IPv6']:
            flowclassifier_id = _uuid()
            tenant_id = _uuid()
            data = {'flow_classifier': {
                'logical_source_port': _uuid(),
                'tenant_id': tenant_id, 'project_id': tenant_id,
                'ethertype': ethertype
            }}
            expected_data = self._get_expected_flow_classifier(data)
            return_value = copy.copy(expected_data['flow_classifier'])
            return_value.update({'id': flowclassifier_id})
            instance = self.plugin.return_value
            instance.create_flow_classifier.return_value = return_value
            res = self.api.post(
                _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt),
                self.serialize(data),
                content_type='application/%s' % self.fmt)
            instance.create_flow_classifier.assert_called_with(
                mock.ANY,
                flow_classifier=expected_data)
            self.assertEqual(exc.HTTPCreated.code, res.status_int)
            res = self.deserialize(res)
            self.assertIn('flow_classifier', res)
            self.assertEqual(return_value, res['flow_classifier'])

    def test_create_flow_classifier_protocol(self):
        for protocol in [
            None, const.PROTO_NAME_TCP, const.PROTO_NAME_UDP,
            const.PROTO_NAME_ICMP
        ]:
            flowclassifier_id = _uuid()
            tenant_id = _uuid()
            data = {'flow_classifier': {
                'logical_source_port': _uuid(),
                'tenant_id': tenant_id, 'project_id': tenant_id,
                'protocol': protocol
            }}
            expected_data = self._get_expected_flow_classifier(data)
            return_value = copy.copy(expected_data['flow_classifier'])
            return_value.update({'id': flowclassifier_id})
            instance = self.plugin.return_value
            instance.create_flow_classifier.return_value = return_value
            res = self.api.post(
                _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt),
                self.serialize(data),
                content_type='application/%s' % self.fmt)
            instance.create_flow_classifier.assert_called_with(
                mock.ANY,
                flow_classifier=expected_data)
            self.assertEqual(exc.HTTPCreated.code, res.status_int)
            res = self.deserialize(res)
            self.assertIn('flow_classifier', res)
            self.assertEqual(return_value, res['flow_classifier'])

    def test_create_flow_classifier_all_fields(self):
        flowclassifier_id = _uuid()
        tenant_id = _uuid()
        data = {'flow_classifier': {
            'name': 'test1',
            'description': 'desc',
            'tenant_id': tenant_id, 'project_id': tenant_id,
            'source_port_range_min': 100,
            'source_port_range_max': 200,
            'destination_port_range_min': 100,
            'destination_port_range_max': 200,
            'l7_parameters': {},
            'destination_ip_prefix': '10.0.0.0/8',
            'source_ip_prefix': '10.0.0.0/8',
            'logical_source_port': _uuid(),
            'logical_destination_port': _uuid(),
            'ethertype': None,
            'protocol': None
        }}
        expected_data = self._get_expected_flow_classifier(data)
        return_value = copy.copy(expected_data['flow_classifier'])
        return_value.update({'id': flowclassifier_id})
        instance = self.plugin.return_value
        instance.create_flow_classifier.return_value = return_value
        res = self.api.post(
            _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)
        instance.create_flow_classifier.assert_called_with(
            mock.ANY,
            flow_classifier=expected_data)
        self.assertEqual(exc.HTTPCreated.code, res.status_int)
        res = self.deserialize(res)
        self.assertIn('flow_classifier', res)
        self.assertEqual(return_value, res['flow_classifier'])

    def test_create_flow_classifier_invalid_l7_parameters(self):
        tenant_id = _uuid()
        data = {'flow_classifier': {
            'logical_source_port': _uuid(),
            'l7_parameters': {'abc': 'def'},
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        self.assertRaises(
            webtest.app.AppError,
            self.api.post,
            _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)

    def test_create_flow_classifier_invalid_protocol(self):
        tenant_id = _uuid()
        data = {'flow_classifier': {
            'logical_source_port': _uuid(),
            'protocol': 'unknown',
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        self.assertRaises(
            webtest.app.AppError,
            self.api.post,
            _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)

    def test_create_flow_classifier_invalid_ethertype(self):
        tenant_id = _uuid()
        data = {'flow_classifier': {
            'logical_source_port': _uuid(),
            'ethertype': 'unknown',
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        self.assertRaises(
            webtest.app.AppError,
            self.api.post,
            _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)

    def test_create_flow_classifier_port_small(self):
        tenant_id = _uuid()
        data = {'flow_classifier': {
            'logical_source_port': _uuid(),
            'source_port_range_min': -1,
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        self.assertRaises(
            webtest.app.AppError,
            self.api.post,
            _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)

    def test_create_flow_classifier_port_large(self):
        tenant_id = _uuid()
        data = {'flow_classifier': {
            'logical_source_port': _uuid(),
            'source_port_range_min': 65536,
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        self.assertRaises(
            webtest.app.AppError,
            self.api.post,
            _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)

    def test_create_flow_classifier_ip_prefix_no_cidr(self):
        tenant_id = _uuid()
        data = {'flow_classifier': {
            'source_ip_prefix': '10.0.0.0',
            'logical_source_port': _uuid(),
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        self.assertRaises(
            webtest.app.AppError,
            self.api.post,
            _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)

    def test_create_flow_classifier_ip_prefix_invalid_cidr(self):
        tenant_id = _uuid()
        data = {'flow_classifier': {
            'source_ip_prefix': '10.0.0.0/33',
            'logical_source_port': _uuid(),
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        self.assertRaises(
            webtest.app.AppError,
            self.api.post,
            _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)

    def test_create_flow_classifier_port_id_nouuid(self):
        tenant_id = _uuid()
        data = {'flow_classifier': {
            'logical_source_port': 'unknown',
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        self.assertRaises(
            webtest.app.AppError,
            self.api.post,
            _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)

    def test_flow_classifier_list(self):
        tenant_id = _uuid()
        flowclassifier_id = _uuid()
        return_value = [{
            'tenant_id': tenant_id, 'project_id': tenant_id,
            'id': flowclassifier_id
        }]
        instance = self.plugin.return_value
        instance.get_flow_classifiers.return_value = return_value

        res = self.api.get(
            _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt))

        instance.get_flow_classifiers.assert_called_with(
            mock.ANY,
            fields=mock.ANY,
            filters=mock.ANY
        )
        self.assertEqual(exc.HTTPOk.code, res.status_int)
        res = self.deserialize(res)
        self.assertIn('flow_classifiers', res)
        self.assertEqual(return_value, res['flow_classifiers'])

    def test_flow_classifier_list_all_fields(self):
        tenant_id = _uuid()
        flowclassifier_id = _uuid()
        return_value = [{
            'name': 'abc',
            'description': 'abc',
            'ethertype': 'IPv4',
            'protocol': const.PROTO_NAME_TCP,
            'source_ip_prefix': '10.0.0.0/8',
            'destination_ip_prefix': '10.0.0.0/8',
            'source_port_range_min': 100,
            'source_port_range_max': 200,
            'destination_port_range_min': 100,
            'destination_port_range_max': 200,
            'logical_source_port': _uuid(),
            'logical_destination_port': _uuid(),
            'l7_parameters': {},
            'tenant_id': tenant_id, 'project_id': tenant_id,
            'id': flowclassifier_id
        }]
        instance = self.plugin.return_value
        instance.get_flow_classifiers.return_value = return_value
        res = self.api.get(
            _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt))
        instance.get_flow_classifiers.assert_called_with(
            mock.ANY,
            fields=mock.ANY,
            filters=mock.ANY
        )
        self.assertEqual(exc.HTTPOk.code, res.status_int)
        res = self.deserialize(res)
        self.assertIn('flow_classifiers', res)
        self.assertEqual(return_value, res['flow_classifiers'])

    def test_flow_classifier_list_unknown_fields(self):
        tenant_id = _uuid()
        flowclassifier_id = _uuid()
        return_value = [{
            'logical_source_port': _uuid(),
            'new_key': 'value',
            'tenant_id': tenant_id, 'project_id': tenant_id,
            'id': flowclassifier_id
        }]
        expected_return = copy.copy(return_value)
        for item in expected_return:
            del item['new_key']
        instance = self.plugin.return_value
        instance.get_flow_classifiers.return_value = return_value
        res = self.api.get(
            _get_path(FLOW_CLASSIFIER_PATH, fmt=self.fmt))
        instance.get_flow_classifiers.assert_called_with(
            mock.ANY,
            fields=mock.ANY,
            filters=mock.ANY
        )
        self.assertEqual(exc.HTTPOk.code, res.status_int)
        res = self.deserialize(res)
        self.assertIn('flow_classifiers', res)
        self.assertEqual(expected_return, res['flow_classifiers'])

    def test_flow_classifier_get(self):
        tenant_id = _uuid()
        flowclassifier_id = _uuid()
        return_value = {
            'logical_source_port': _uuid(),
            'tenant_id': tenant_id, 'project_id': tenant_id,
            'id': flowclassifier_id
        }
        instance = self.plugin.return_value
        instance.get_flow_classifier.return_value = return_value
        res = self.api.get(
            _get_path(
                FLOW_CLASSIFIER_PATH,
                id=flowclassifier_id, fmt=self.fmt
            )
        )
        instance.get_flow_classifier.assert_called_with(
            mock.ANY,
            flowclassifier_id,
            fields=mock.ANY
        )
        self.assertEqual(exc.HTTPOk.code, res.status_int)
        res = self.deserialize(res)
        self.assertIn('flow_classifier', res)
        self.assertEqual(return_value, res['flow_classifier'])

    def test_flow_classifier_update(self):
        tenant_id = _uuid()
        flowclassifier_id = _uuid()
        update_data = {'flow_classifier': {
            'name': 'new_name',
            'description': 'new_desc',
        }}
        return_value = {
            'tenant_id': tenant_id, 'project_id': tenant_id,
            'id': flowclassifier_id
        }

        instance = self.plugin.return_value
        instance.update_flow_classifier.return_value = return_value

        res = self.api.put(
            _get_path(FLOW_CLASSIFIER_PATH, id=flowclassifier_id,
                      fmt=self.fmt),
            self.serialize(update_data))

        instance.update_flow_classifier.assert_called_with(
            mock.ANY, flowclassifier_id,
            flow_classifier=update_data)
        self.assertEqual(exc.HTTPOk.code, res.status_int)
        res = self.deserialize(res)
        self.assertIn('flow_classifier', res)
        self.assertEqual(return_value, res['flow_classifier'])

    def test_flow_classifier_update_source_port_range_min(self):
        tenant_id = _uuid()
        flowclassifier_id = _uuid()
        data = {'flow_classifier': {
            'source_port_range_min': 100,
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        self.assertRaises(
            webtest.app.AppError,
            self.api.put,
            _get_path(FLOW_CLASSIFIER_PATH, id=flowclassifier_id,
                      fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)

    def test_flow_classifier_update_source_port_range_max(self):
        tenant_id = _uuid()
        flowclassifier_id = _uuid()
        data = {'flow_classifier': {
            'source_port_range_max': 100,
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        self.assertRaises(
            webtest.app.AppError,
            self.api.put,
            _get_path(FLOW_CLASSIFIER_PATH, id=flowclassifier_id,
                      fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)

    def test_flow_classifier_update_destination_port_range_min(self):
        tenant_id = _uuid()
        flowclassifier_id = _uuid()
        data = {'flow_classifier': {
            'destination_port_range_min': 100,
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        self.assertRaises(
            webtest.app.AppError,
            self.api.put,
            _get_path(FLOW_CLASSIFIER_PATH, id=flowclassifier_id,
                      fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)

    def test_flow_classifier_update_destination_port_range_max(self):
        tenant_id = _uuid()
        flowclassifier_id = _uuid()
        data = {'flow_classifier': {
            'destination_port_range_max': 100,
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        self.assertRaises(
            webtest.app.AppError,
            self.api.put,
            _get_path(FLOW_CLASSIFIER_PATH, id=flowclassifier_id,
                      fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)

    def test_flow_classifier_update_source_ip_prefix(self):
        tenant_id = _uuid()
        flowclassifier_id = _uuid()
        data = {'flow_classifier': {
            'source_ip_prefix': '10.0.0.0/8',
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        self.assertRaises(
            webtest.app.AppError,
            self.api.put,
            _get_path(FLOW_CLASSIFIER_PATH, id=flowclassifier_id,
                      fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)

    def test_flow_classifier_update_destination_ip_prefix(self):
        tenant_id = _uuid()
        flowclassifier_id = _uuid()
        data = {'flow_classifier': {
            'destination_ip_prefix': '10.0.0.0/8',
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        self.assertRaises(
            webtest.app.AppError,
            self.api.put,
            _get_path(FLOW_CLASSIFIER_PATH, id=flowclassifier_id,
                      fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)

    def test_flow_classifier_update_logical_source_port(self):
        tenant_id = _uuid()
        flowclassifier_id = _uuid()
        data = {'flow_classifier': {
            'logical_source_port': _uuid(),
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        self.assertRaises(
            webtest.app.AppError,
            self.api.put,
            _get_path(FLOW_CLASSIFIER_PATH, id=flowclassifier_id,
                      fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)

    def test_flow_classifier_update_logical_destination_port(self):
        tenant_id = _uuid()
        flowclassifier_id = _uuid()
        data = {'flow_classifier': {
            'logical_destination_port': _uuid(),
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        self.assertRaises(
            webtest.app.AppError,
            self.api.put,
            _get_path(FLOW_CLASSIFIER_PATH, id=flowclassifier_id,
                      fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)

    def test_flow_classifier_update_ethertype(self):
        tenant_id = _uuid()
        flowclassifier_id = _uuid()
        data = {'flow_classifier': {
            'ethertype': None,
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        self.assertRaises(
            webtest.app.AppError,
            self.api.put,
            _get_path(FLOW_CLASSIFIER_PATH, id=flowclassifier_id,
                      fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)

    def test_flow_classifier_update_protocol(self):
        tenant_id = _uuid()
        flowclassifier_id = _uuid()
        data = {'flow_classifier': {
            'protococol': None,
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        self.assertRaises(
            webtest.app.AppError,
            self.api.put,
            _get_path(FLOW_CLASSIFIER_PATH, id=flowclassifier_id,
                      fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)

    def test_flow_classifier_update_l7_parameters(self):
        tenant_id = _uuid()
        flowclassifier_id = _uuid()
        data = {'flow_classifier': {
            'l7_parameters': {},
            'tenant_id': tenant_id, 'project_id': tenant_id,
        }}
        self.assertRaises(
            webtest.app.AppError,
            self.api.put,
            _get_path(FLOW_CLASSIFIER_PATH, id=flowclassifier_id,
                      fmt=self.fmt),
            self.serialize(data),
            content_type='application/%s' % self.fmt)

    def test_flow_classifier_delete(self):
        self._test_entity_delete('flow_classifier')



# Copyright (c) 2015 Huawei Technologies India Pvt.Limited.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutronclient.common import extension
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronv20

from networking_sfc._i18n import _
from networking_sfc.cli import flow_classifier as fc
from networking_sfc.cli import port_pair_group as ppg

PORT_CHAIN_RESOURCE = 'port_chain'


class PortChain(extension.NeutronClientExtension):
    resource = PORT_CHAIN_RESOURCE
    resource_plural = '%ss' % resource
    object_path = '/sfc/%s' % resource_plural
    resource_path = '/sfc/%s/%%s' % resource_plural
    versions = ['2.0']


class PortChainCreate(extension.ClientExtensionCreate, PortChain):
    """Create a Port Chain."""

    shell_command = 'port-chain-create'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name',
            metavar='NAME',
            help=_('Name of the Port Chain.'))
        parser.add_argument(
            '--description',
            help=_('Description for the Port Chain.'))
        parser.add_argument(
            '--port-pair-group',
            metavar='PORT-PAIR-GROUP',
            dest='port_pair_groups',
            default=[], required=True,
            action='append',
            help=_('ID or name of the Port Pair Group. '
                   'This option can be repeated.'))
        parser.add_argument(
            '--flow-classifier',
            default=[],
            metavar='FLOW-CLASSIFIER',
            dest='flow_classifiers',
            action='append',
            help=_('ID or name of the Flow Classifier.'
                   'This option can be repeated.'))
        parser.add_argument(
            '--chain-parameters',
            metavar='[correlation=CORRELATION_TYPE, symmetric=BOOLEAN_TYPE]',
            type=utils.str2dict_type(optional_keys=['correlation',
                                                    'symmetric']),
            help=_('Dictionary of chain parameters. Supports '
                   'correlation=mpls and symmetric=true|false.'))

    def args2body(self, parsed_args):
        body = {}
        client = self.get_client()
        if parsed_args.port_pair_groups:
            body['port_pair_groups'] = [ppg.get_port_pair_group_id(client, p)
                                        for p in parsed_args.port_pair_groups]
        if parsed_args.flow_classifiers:
            body['flow_classifiers'] = [fc.get_flowclassifier_id(client, f)
                                        for f in parsed_args.flow_classifiers]
        neutronv20.update_dict(parsed_args, body, ['name', 'description',
                                                   'chain_parameters'])
        return {self.resource: body}


class PortChainUpdate(extension.ClientExtensionUpdate, PortChain):
    """Update Port Chain's information."""

    shell_command = 'port-chain-update'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--name',
            metavar='NAME',
            help=_('Name of the Port Chain.'))
        parser.add_argument(
            '--description',
            help=_('Description for the Port Chain.'))
        fw_args = parser.add_mutually_exclusive_group()
        fw_args.add_argument(
            '--flow-classifier',
            metavar='FLOW-CLASSIFIER',
            dest='flow_classifiers',
            action='append',
            help=_('ID or name of the Flow Classifier. '
                   'This option can be repeated.'))
        fw_args.add_argument(
            '--no-flow-classifier',
            action='store_true',
            help=_('Associate no Flow Classifier with the Port Chain.'))
        parser.add_argument(
            '--port-pair-group',
            metavar='PORT-PAIR-GROUP',
            dest='port_pair_groups',
            action='append',
            help=_('ID or name of the port pair group. '
                    'This option can be repeated.'))

    def args2body(self, parsed_args):
        body = {}
        client = self.get_client()
        if parsed_args.flow_classifiers:
            body['flow_classifiers'] = [fc.get_flowclassifier_id(client, f)
                                        for f in parsed_args.flow_classifiers]
        elif parsed_args.no_flow_classifier:
            body['flow_classifiers'] = []
        if parsed_args.port_pair_groups:
            body['port_pair_groups'] = [ppg.get_port_pair_group_id(client, p)
                                        for p in parsed_args.port_pair_groups]
        neutronv20.update_dict(parsed_args, body, ['name', 'description'])
        return {self.resource: body}


class PortChainDelete(extension.ClientExtensionDelete, PortChain):
    """Delete a given Port Chain."""

    shell_command = 'port-chain-delete'


class PortChainList(extension.ClientExtensionList, PortChain):
    """List Port Chains that belong to a given tenant."""

    shell_command = 'port-chain-list'
    list_columns = ['id', 'name', 'port_pair_groups', 'flow_classifiers']
    pagination_support = True
    sorting_support = True


class PortChainShow(extension.ClientExtensionShow, PortChain):
    """Show information of a given Port Chain."""

    shell_command = 'port-chain-show'
